#!/usr/local/bin/perl
#
#       dmProxy.pl : proxy d'un serveur DMeroon
# ---------------------------------------------------
# Application 	: dmProxy.pl
# Fichier 	: dmProxy.pl
# Auteur 	: JM Inglebert - IUT de Vannes
# Mail 	: Jean-Michel.Inglebert@iu-vannes.fr
# Date	: 29/03/98
# ---------------------------------------------------
  sub rcsId { return '$Id: dmProxy.pl,v 1.2 1998/05/01 07:49:19 queinnec Exp $' }
# ---------------------------------------------------
## TODO
## - chargement de fichier urlscript
## - requete GET ou POST pure
## TODO

# ======================================================================
sub usage {
 print <<"END"
  Get requests from an http client and sends them to the 
  specified DMeroon server.
  Translate GET requests to URLscript requests before sending them.
  Put responses on the http client socket
END
}
# ======================================================================

# le package socket
use Socket;

# arguments ?
($proxyPort,$DMserver,$DMport) = @ARGV;

# service proxy
$proxyPort = 9876 unless $proxyPort;

# service DMeroon
$DMserver = "pc-valoria-7.univ-ubs.fr" unless $DMserver;
$DMport   = 56423                      unless $DMport;

# request number
$rqNb = 0 ;
$anchorString = "" ;


#
# Param'trage du socket d''coute
#
# le format de la structure C sock_addr
 $formatAddr = 'S n a4 x8';

# Qui suis-je ?
 chop($hostname = `hostname`) ;

# comment dit-on 'tcp' ?
 ($name,$aliases,$proto) = getprotobyname('tcp');

# on fabrique la structure C af_inet qui d'finit
# le service et son num'ro (port).
 $strServ = pack($formatAddr, &AF_INET, $proxyPort, "\0\0\0\0") ;

# pourrai-je avoir un socket 'tcp' ?
 socket(ECOUTE, &PF_INET, &SOCK_STREAM, $proto) ||
  die "erreur : socket";

# qui impl'menterai le service $strServ
 bind(ECOUTE,$strServ) ||
  die "erreur : bind socket sur $proxyPort";

# et qui accepterai au plus 3 requStes simultan'es
 listen(ECOUTE,3) ||
  die "erreur : listen socket";

# Le service est operationnel 
 print "Service $0 sur le port $proxyPort de la machine $hostname actif\n";


# indefiniement ...
for (;;) {

 # on attend les clients ...
 accept(CONNEXION,ECOUTE)||
  die "erreur : accept socket";

 # on d'bufferise les E/S sur le socket et STDERR
 select CONNEXION ; $|=1 ; select STDERR ; $|=1 ;

 print STDERR  "Connexion d'un client\n";

# mode echo sur STDOUT si ( $DMserver eq "-" )
if ( $DMserver eq "-" ) 
{
 *DMCONNECT = *STDOUT ;
 select STDOUT ; $|=1 ;
 print STDERR "No Server mode : no response\n"  ;
}
else 
{
# Connexion DMeroon
# -----------------
# quelle est l'adresse IP du serveur
 ($name,$aliases,$type,$len,$dmIP) = gethostbyname($DMserver);
# on fabrique la structure C af_inet qui d'finit le service
 $strService = pack($formatAddr, &AF_INET, $DMport, $dmIP) ;

# pourrai-je avoir un socket 'tcp' ?
 socket(DMCONNECT, &PF_INET, &SOCK_STREAM, $proto) ||
  die "erreur : cr'ation socket";

# qui serait connect' au service $strService
 connect(DMCONNECT,$strService) ||
  die "erreur : connection socket";

# 
print STDERR "Connexion 'tcp' avec $DMserver sur le port $DMport\n";
}

# on d'bufferise les E/S sur DMCONNECT
 select DMCONNECT; $|=1; select STDOUT ;


# on lit la requSte dans un fils
 $PID=fork;
 if ($PID == 0) 
  {
    # FUTURE : blinder pour les formats exotiques !

    # on saute tout jusqu'. 'FORM URLscript'
    do
     {
      $_ = <CONNEXION> ; 
      # print STDERR $_ ;
     } until /FORM URLscript ([0-9]*)/i ;

    # gestion des num'ros de requ^tes
    $rqNb = $1 ;
    $anchorString = "<A NAME=\"" . $rqNb . "\"> <B> Request Number : " . $rqNb . " </B>" ;

    # connexion DMeroon : protocole URLscript
    print DMCONNECT "URLscript\r\n" ;
    # on lit la requete
    do
     {
      $_ = <CONNEXION> ;

      # et on la transmet aussit"t
      print DMCONNECT $_ unless /FINFORM URLscript/i;
     } until /FINFORM URLscript/i ;

    # fin du protocole URLscript
    print DMCONNECT "\0\0\r\n" ;

    exit(0);
  }

# retour dans le pSre
if ( $DMserver eq "-" ) 
{
 print CONNEXION "No Server mode : no response \n"  ;
 close CONNEXION ;
}
else
{
# on lit la r'ponse de DMeroon
 while(<DMCONNECT>)
     {
      $response = $response . $_;
     }
 close DMCONNECT || die "pb close" ;
 print CONNEXION $anchorString ;
 print CONNEXION $response ;
 close CONNEXION ;
}

} # fin for ...


# ==========================================================
# perl documentation
# ==========================================================
=head1  NAME

dmProxy.pl - URLscript Proxy server for DMeroon servers

=head1  SYNOPSIS

dmProxy.pl [I<proxyServiceName>]  [I<DMeroonServerName>]  [I<DMeroonServiceName>]

=head1  EXAMPLES

For debug mode with echo on STDERR :

	perl dmProxy.pl 9876 -

Default local host and port :

	perl dmProxy.pl

Proxy on localhost:9999 and DMeroon service on zeus.iu-vannes.fr:56014

	perl dmProxy.pl 9999 zeus.iu-vannes.fr 56014


=head1 DESCRIPTION

Run a proxy server on F<localhost:proxyServiceName> which
sends requests to the DMeroon service F<DMeroonServerName:DMeroonServiceName>.

=head1 AUTHOR

F<Jean-Michel.Inglebert@univ-ubs.fr> UBS - VALORIA/ORCADE

=head1 SEE ALSO

DMeroon home page	http://www-spi.lip6.fr/~queinnec/WWW/DMeroon.html

F<Christian.Queinnec@acm.org> for remarks etc on DMeroon

=cut

