/* $Id$ 
 **************************************************************************
 envoi.c
 
 Envoi d'un point vers un site distant.
 Yves Maheo.
**************************************************************************/

#include "dmeroon.h"

#define OTHER_HOST "localhost"
#define OTHER_PORT 56221
#define PASSWD NULL

int
main() {
  DM_Context *ctx = NULL;
  DM_Site *site_ici, *site_labas;           
  DM_ErrorCode errcode;
  
  DM_Class  *Point_class, *px, *py ;
  DM_Field  *point_abscisse, *point_ordonnee ;
 
  DM_Object *Sommet;

  int initx, inity ;     /* valeur initiale du sommet */
  int valeurx, valeury;  /* valeur courante du sommet */

  /* ---------- Initialisation de DMeroon */
  errcode = DMeroon_initialize(ctx, &site_ici);
  if ( errcode != DME_SUCCESS ) {
    fprintf(DM_debug_stream, "\tCannot initialize DMeroon (error=%lu).\n",
            (unsigned long)errcode );
    exit(5);
  }
  site_ici->debug_level = 40;
  printf("Serveur Dmeroon lanc. Appuyez sur une touche ");
  { char rep ; scanf("%c",&rep); }
  
  /* ----------- Donnees initiales  */ 
  printf("Donnez 2 entiers : "); 
  scanf("%d%d", &initx, &inity);

  /* ----------- Creation de la classe Point */ 
  (void)DMeroon_create_subclass(ctx, &px, DM_ReferenceToClass(Object),
                                "class1", DM_CLASS_DEFAULT_OPTIONS,
                                "abscisse", DM_FIELD_IS_MUTABLE,
                                DM_ReferenceToDMeroonType(nought),
                                DM_ReferenceToDMeroonType(int3),
                                DM_NULL);
  (void)DMeroon_create_subclass(ctx, &py, px,
                                "class2", DM_CLASS_DEFAULT_OPTIONS,
                                "ordonnee", DM_FIELD_IS_MUTABLE,
                                DM_ReferenceToDMeroonType(nought),
                                DM_ReferenceToDMeroonType(int3),
                                DM_NULL);
  (void)DMeroon_smash_class(ctx, &Point_class, py, "Point",
                            DM_CLASS_DEFAULT_OPTIONS, 1);
 

  /* ----------- Creation des acces aux champs de la classe Point */
  (void)DMeroon_indexed_field_value(ctx, &point_abscisse, DM_NULL,
                                    (DM_Object *)Point_class,
                                    DM_ReferenceToFieldOfClass(field, Class),
                                    0);
  (void)DMeroon_indexed_field_value(ctx, &point_ordonnee, DM_NULL,
                                    (DM_Object *)Point_class,
                                    DM_ReferenceToFieldOfClass(field, Class),
                                    1);

  /* ------------ Allocation de l'objet Sommet */
  (void)DMeroon_allocate(ctx, &Sommet, Point_class, 0, 0);

  /* ------------ Initialisation de l'objet Sommet */
  (void)DMeroon_set_field_value(ctx, NULL, Sommet, point_abscisse, &initx);
  (void)DMeroon_set_field_value(ctx, NULL, Sommet, point_ordonnee, &inity);
  (void)DMeroon_set_initialized(ctx, Sommet);
  
  /* ----------- Affichage des champs de l'objet Sommet */
  (void)DMeroon_field_value(ctx, &valeurx, NULL, Sommet, point_abscisse);
  (void)DMeroon_field_value(ctx, &valeury, NULL, Sommet, point_ordonnee);
  printf("point = (%d,%d)\n", valeurx, valeury);

  /* ----------- Connection avec la machine distante */
  printf("Connecting to " OTHER_HOST  " ...\n"); 
  errcode = DMeroon_connect(ctx, &site_labas, OTHER_HOST, OTHER_PORT, PASSWD);
  if ( errcode != DME_SUCCESS ) {
    fprintf(DM_debug_stream, "\tCannot connect to " OTHER_HOST "(error=%lu).\n",
            (unsigned long)errcode );
    exit(5);
  }
  /* ************* Segmentation fault 120sec aprs l'appel  connect */
  printf("Connexion avec " OTHER_HOST " OK\n"); fflush(stdout);

  /* ---------- Envoi du Sommet sur la machine distante */
  errcode = DMeroon_send(ctx, Sommet, site_labas);
  if ( errcode != DME_SUCCESS ) {
    fprintf(DM_debug_stream, "\tCannot send to " OTHER_HOST "(error=%lu).\n",
            (unsigned long)errcode );
    exit(5);
  }
  printf("Envoi vers " OTHER_HOST " OK\n");

  /* ----------- On donne la main 5 sec au serveur */
  (void)DMeroon_serve(ctx, 5);

  return EXIT_SUCCESS;
}

/* end of envoi.c */
