/* $Id: dmpico.h,v 1.3 1995/12/15 09:02:16 queinnec Exp $
 * ///////////////////////////////////////////////////////////////////////
 *                    //          //
 *                   // DMeroon  //
 *                  //          //
 * A continuation-based causal distributed class-based object system.
 * Christian.Queinnec@polytechnique.fr
 * ///////////////////////////////////////////////////////////////////////
 */

#ifndef DMeroon_PICO_H
#define DMeroon_PICO_H

#define DM_SUPPORT_LANGUAGE "Pico"

/* 
 * This is the Pico header file for DMeroon.
 */

#include "pico.h"

typedef struct pico_struct {
  DM_nat3 _header;
  DM_nat3 _size;
} pico_struct;

typedef struct pico_port {
  struct     pico_struct;
  DM_nat3    tag;
  pico_struct *name;
  FILE       *file;
} pico_port;

typedef struct pico_double {
  struct     pico_struct;
  double     value;
} pico_double;

/******************************************************************
 * This is the value used in DMeroon objects header for Pico.
 */

#define DM_HEADER pico_DMeroon_header

/* 
 * This is the macro that recognizes a Pico object as a DMeroon instance.
 * NOTE: Attention tout ce qui suit depend fortement de 32 bits. 
 */ 

#define PicoRemoveTag(pico,tag) ((void*) (((DM_SupportAddress)(pico))-tag))
#define PicoInsertTag(p,tag)    ((DM_SupportPointer) (((DM_SupportAddress)(p))|tag))
#define PicoExtractTag(pico)    (((DM_SupportAddress)(pico))&3)

#define PICO_header(pico) (((pico_struct*)(PicoRemoveTag((pico),2)))->_header)

#define PICO_structurep(pico) (PicoExtractTag(pico) == 2)

#define PICO_TAG_SHIFT 2

#define PICO_FLOAT_HEADER pico_float_header

#define DM_IsDMeroonInstance(pico) \
  ( PICO_structurep(pico) && (PICO_header(pico) == DM_HEADER) )

/*
 * Here are the types of the various kinds of values handled by Pico.
 * DM_SupportBoolean is only for advertizing predicates.
 */

#define DM_SupportPointer  PICO_OBJ
#define DM_SupportFixnum   long
#define DM_SupportFloatnum double
#define DM_SupportBoolean  DM_SupportPointer
#define DM_SupportAddress  long
#define DM_SupportVoid     DM_SupportPointer

/* 
 * This is how Pico values are recognized according to their types.
 */

#define DM_IsSupportFixnum(wrapped_pico) \
  (PicoExtractTag(wrapped_pico) == 0)
#define DM_IsSupportCharacter(wrapped_pico) \
  ((((DM_SupportAddress)(wrapped_pico))&255) == 7)
#define DM_IsSupportFloat(wrapped_o) \
  DM_IsSupportDouble(wrapped_o)
#define DM_IsSupportDouble(wrapped_o) \
  (PICO_structurep(wrapped_o) && (PICO_header(wrapped_o) == PICO_FLOAT_HEADER ))
#define DM_IsSupportLongDouble(wrapped_o) \
  DM_IsSupportDouble(wrapped_o)

/* 
 * This is how Pico values are built.
 */

#define DM_ToSupport(dmeroon) \
  ( PicoInsertTag((((DMeroon_object*) (dmeroon)) - 1), 2) )
#define DM_ToSupportFixnum(cint) \
  ((DM_SupportPointer)(((long)(cint))<<2))
#define DM_ToSupportCharacter(cchar) \
  ((DM_SupportPointer)(((cchar)<<8)+7))
#define DM_ToSupportFloat(float) \
  DM_ToSupportDouble((double)float)
#define DM_ToSupportDouble(double) \
  ( PICO_allocate_double(double) )
#define DM_ToSupportLongDouble(longdouble) \
  DM_ToSupportDouble((double)longdouble)

/*
 * This is how Pico values are decoded.
 */

#define DM_FromSupport(pico) \
  ( (void*)(((DMeroon_object*) (PicoRemoveTag(pico, 2))) + 1) )
#define DM_FromSupportFixnum(wrapped_int) \
  (((DM_SupportAddress)wrapped_int)>>2)
#define DM_FromSupportCharacter(wrapped_char) \
  (((DM_SupportAddress)wrapped_char)>>8)
#define DM_FromSupportFloat(wrapped_float) \
  DM_FromSupportDouble(wrapped_float)
#define DM_FromSupportDouble(wrapped_double) \
  0.0
#define DM_FromSupportLongDouble(wrapped_longdouble) \
  DM_FromSupportDouble(wrapped_longdouble)

/*
 * DM_FALSE and DM_TRUE designates the Pico representations of #f and #t.
 */

#define DM_FALSE ((DM_SupportPointer) pico_false)
#define DM_TRUE  ((DM_SupportPointer) pico_true)
#define DM_DONE  DM_TRUE


#define DM_SupportPrefix() \
  DM_nat3        _header; \
  DM_nat3        _size;

#define DM_SupportPrefixInitialization(type, varname) \
  DM_HEADER,\
  sizeof(type),

/*
 * Limits set by Pico.
 * DM_SUPPORT_FIXNUM_MAXIMUM is the most positive small integer Pico 
 * can represent. DM_SUPPORT_FIXNUM_MINIMUM is the least negative one
 * and must be defined as its opposite.

 * On a 32bits cpu with a 2bits tag, this yields:
 *   DM_SUPPORT_FIXNUM_MAXIMUM = 536870911
 * Observe that we are insensitive to integer representation such as
 * complement to 1 or 2 (but we loose one integer!).
 * On a 64bits cpu with a 3bits tag, this yields:
 *   DM_SUPPORT_FIXNUM_MAXIMUM = 1152921504606846976

 * This definition is correct but cannot be used in #if statement since
 * it contains casts and sizeof operators. I therefore rewrite it a little
 * to be more static: DM_SupportFixnum is int_t which is currently long.
 * This assumption is checked in DM_initialize (dmbgl.c).
 *#define DM_SUPPORT_FIXNUM_MAXIMUM \
 * ((DM_SupportFixnum) \
 * ((((DM_SupportFixnum)1)<<( (CHAR_BIT*sizeof(DM_SupportFixnum)) \
 *                           - TAG_SHIFT -1 ))-1))
 */

#define DM_SUPPORT_FIXNUM_MAXIMUM (536870911)

#define DM_SUPPORT_FIXNUM_MINIMUM (-DM_SUPPORT_FIXNUM_MAXIMUM)

#undef ULONG_LONG_MAX
#undef LONG_LONG_MAX

/******************************************************************
 *                      FUNCTIONS
 */

extern DM_SupportPointer 
DM_allocate_bare_instance (DM_SupportPointer wrapped_class, 
                           DM_nat3           size,
                           DM_SupportPointer wrapped_reference );

extern void
DM_set_exit_finalization (DM_SupportPointer wrapped_exit);

extern DM_SupportPointer
DM_raise_exception (DM_nat3 code, 
                    char    *message, 
                    char    *file, 
                    DM_nat3 line);

extern DM_SupportBoolean
DM_print_char_star (char *string, 
		    DM_nat3 len,
		    DM_SupportPointer wrapped_port);

extern DM_SupportPointer
DM_initialize (void);

/*                                  A FOURNIR
 *
 * extern PICO_allocate (taille multiple de 4) -> void*  
 * extern PICO_output_portp(pico) -> Cbool
 * extern PICO_error(dmeroon-pico)
 */

extern PICO_OBJ PICO_allocate_double(double d);

#endif /* DMeroon_Pico_H */

/* end of dmpico.h */
